@@ -1,7 +1,3 @@
-#ifdef INCRCSDATA
-static char RCSid[]="$Id: print.c,v 1.4 2005/01/04 13:01:38 mikulik Exp $" ;
-#endif
-
 /****************************************************************************
 
     PROGRAM: gnupmdrv
