@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: tables.c,v 1.155 2017/05/20 01:27:30 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - tables.c */
 
 /*[
@@ -44,7 +40,8 @@ static char *RCSid() { return RCSid("$Id: tables.c,v 1.155 2017/05/20 01:27:30 s
 #include "util.h"
 #include "alloc.h"	/* for init_colornames() */
 #include "graph3d.h"	/* for DGRID3D_* options */
-# include "getcolor.h"
+#include "getcolor.h"
+#include "voxelgrid.h"
 
 /* gnuplot commands */
 
@@ -96,6 +93,9 @@ const struct gen_ftable command_ftbl[] =
     { "und$efine", undefine_command },
     { "uns$et", unset_command },
     { "up$date", update_command },
+    { "vclear", vclear_command },
+    { "vfill", vfill_command },
+    { "voxel", voxel_command },
     { "while", while_command },
     { "{", begin_clause },
     { "}", end_clause },
@@ -131,6 +131,7 @@ const struct gen_table plot_smooth_tbl[] =
     { "cn$ormal", SMOOTH_CUMULATIVE_NORMALISED },
     { "mcs$plines", SMOOTH_MONOTONE_CSPLINE },
     { "fnor$mal", SMOOTH_FREQUENCY_NORMALISED },
+    { "z$sort", SMOOTH_ZSORT },
     { NULL, SMOOTH_NONE }
 };
 
@@ -170,6 +171,7 @@ const struct gen_table set_tbl[] =
     { "b$ars", S_BARS },
     { "bind", S_BIND },
     { "bor$der", S_BORDER },
+    { "boxdepth", S_BOXDEPTH },
     { "box$width", S_BOXWIDTH },
     { "cl$abel", S_CLABEL },
     { "c$lip", S_CLIP },
@@ -182,6 +184,7 @@ const struct gen_table set_tbl[] =
 
     { "data$file", S_DATAFILE },
 
+    { "debug", S_DEBUG },
     { "dg$rid3d", S_DGRID3D },
     { "du$mmy", S_DUMMY },
     { "enc$oding", S_ENCODING },
@@ -196,6 +199,8 @@ const struct gen_table set_tbl[] =
     { "hid$den3d", S_HIDDEN3D },
     { "historysize", S_HISTORYSIZE },	/* Deprecated */
     { "his$tory", S_HISTORY },
+    { "pixmap$s", S_PIXMAP },
+    { "isosurf$ace", S_ISOSURFACE },
     { "is$osamples", S_ISOSAMPLES },
     { "jitter", S_JITTER },
     { "k$ey", S_KEY },
@@ -221,9 +226,7 @@ const struct gen_table set_tbl[] =
 
     { "micro", S_MICRO },
     { "minus$sign", S_MINUS_SIGN },
-#ifdef USE_MOUSE
     { "mo$use", S_MOUSE },
-#endif
     { "mono$chrome", S_MONOCHROME },
     { "multi$plot", S_MULTIPLOT },
 
@@ -245,6 +248,7 @@ const struct gen_table set_tbl[] =
     { "of$fsets", S_OFFSETS },
     { "or$igin", S_ORIGIN },
     { "o$utput", SET_OUTPUT },
+    { "overflow", S_OVERFLOW },
     { "pa$rametric", S_PARAMETRIC },
     { "pm$3d", S_PM3D },
     { "pal$ette", S_PALETTE },
@@ -259,10 +263,13 @@ const struct gen_table set_tbl[] =
     { "pr$int", S_PRINT },
     { "psdir", S_PSDIR },
     { "obj$ect", S_OBJECT },
+    { "wall$s", S_WALL },
+    { "rgbmax", S_RGBMAX },
     { "sa$mples", S_SAMPLES },
     { "si$ze", S_SIZE },
     { "st$yle", S_STYLE },
     { "su$rface", S_SURFACE },
+    { "spider$plot", S_SPIDERPLOT },
     { "table", S_TABLE },
     { "t$erminal", S_TERMINAL },
     { "termopt$ions", S_TERMOPTIONS },
@@ -271,7 +278,7 @@ const struct gen_table set_tbl[] =
     { "ticsc$ale", S_TICSCALE },
     { "ticsl$evel", S_TICSLEVEL },
     { "timef$mt", S_TIMEFMT },
-    { "tim$estamp", S_TIMESTAMP },
+    { "times$tamp", S_TIMESTAMP },
     { "tit$le", S_TITLE },
     { "v$ariables", S_VARIABLES },
     { "ve$rsion", S_VERSION },
@@ -345,6 +352,11 @@ const struct gen_table set_tbl[] =
     { "ur$ange", S_URANGE },
     { "vr$ange", S_VRANGE },
 
+    { "vgrid", S_VGRID },
+    { "vxr$ange", S_VXRANGE },
+    { "vyr$ange", S_VYRANGE },
+    { "vzr$ange", S_VZRANGE },
+
     { "xzeroa$xis", S_XZEROAXIS },
     { "x2zeroa$xis", S_X2ZEROAXIS },
     { "yzeroa$xis", S_YZEROAXIS },
@@ -485,8 +497,6 @@ const struct gen_table color_model_tbl[] =
     { "RGB", C_MODEL_RGB },
     { "HSV", C_MODEL_HSV },
     { "CMY", C_MODEL_CMY },
-    { "YIQ", C_MODEL_YIQ },
-    { "XYZ", C_MODEL_XYZ },
     { NULL,  -1 }
 };
 
@@ -502,8 +512,11 @@ const struct gen_table set_pm3d_tbl[] =
     { "fl$ush",		S_PM3D_FLUSH },
     { "ftr$iangles",	S_PM3D_FTRIANGLES },
     { "noftr$iangles",	S_PM3D_NOFTRIANGLES },
+    { "clip$z", 	S_PM3D_CLIP_Z },
     { "clip1$in",	S_PM3D_CLIP_1IN },
     { "clip4$in",	S_PM3D_CLIP_4IN },
+    { "clipcb", 	S_PM3D_CLIPCB },
+    { "noclipcb",	S_PM3D_NOCLIPCB },
     { "map", 		S_PM3D_MAP },
     { "bo$rder",	S_PM3D_BORDER },
     { "nobo$rder",	S_PM3D_NOBORDER },
@@ -674,9 +687,8 @@ const struct gen_table show_style_tbl[] =
     { "rect$angle", SHOW_STYLE_RECTANGLE },
     { "boxplot", SHOW_STYLE_BOXPLOT },
     { "parallel$axis", SHOW_STYLE_PARALLEL },
-#ifdef EAM_BOXED_TEXT
+    { "spider$plot", SHOW_STYLE_SPIDERPLOT },
     { "textbox", SHOW_STYLE_TEXTBOX },
-#endif
     { NULL, SHOW_STYLE_INVALID }
 };
 
@@ -698,6 +710,7 @@ const struct gen_table plotstyle_tbl[] =
     { "xye$rrorbars", XYERRORBARS },
     { "boxes", BOXES },
     { "hist$ograms", HISTOGRAMS },
+    { "isosurface", ISOSURFACE },
     { "filledc$urves", FILLEDCURVES },
     { "boxer$rorbars", BOXERROR },
     { "boxx$yerrorbars", BOXXYERROR },
@@ -706,20 +719,21 @@ const struct gen_table plotstyle_tbl[] =
     { "fs$teps", FSTEPS },
     { "his$teps", HISTEPS },
     { "vec$tors", VECTOR },
+    { "arrow$s", ARROWS },
     { "fin$ancebars", FINANCEBARS },
     { "can$dlesticks", CANDLESTICKS },
     { "boxplot", BOXPLOT },
     { "pm$3d", PM3DSURFACE },
+    { "poly$gons", POLYGONS },
     { "labels", LABELPOINTS },
     { "ima$ge", IMAGE },
     { "rgbima$ge", RGBIMAGE },
     { "rgba$lpha", RGBA_IMAGE },
-#ifdef EAM_OBJECTS
     { "cir$cles", CIRCLES },
     { "ell$ipses", ELLIPSES },
-#endif
     { "sur$face", SURFACEGRID },
     { "parallel$axes", PARALLELPLOT },
+    { "spider$plot", SPIDERPLOT },
     { "table", TABLESTYLE },
     { "zerror$fill", ZERRORFILL },
     { NULL, PLOT_STYLE_NONE }
@@ -740,6 +754,16 @@ const struct gen_table filledcurves_opts_tbl[] =
     { NULL, -1 }
 };
 
+const struct gen_table fit_verbosity_level[] =
+{
+    { "brief", BRIEF },
+    { "quiet", QUIET },
+    { "noquiet", BRIEF },
+    { "results", RESULTS },
+    { "verbose", VERBOSE },
+    { NULL, -1 }
+};
+
 int
 lookup_table(const struct gen_table *tbl, int find_token)
 {
